@@ -1,22 +1,32 @@
-/****************************************************************************************/
-/*											*/
-/* Sentry - Port Scan Detector IO Library						*/
-/*											*/
-/* Author: Craig H. Rowland <crowland@psionic.com> <crowland@bipolar.net>		*/
-/* Created: 10-12-97									*/
-/*											*/
-/* This software is Copyright(c) 1997-98 Craig H. Rowland				*/
-/*											*/
-/* This software is free to use provided the enclosed DISCLAIMER is read and adhered	*/
-/* to. Use of this software in a commercial product offering is prohibited.		*/
-/* Code changes are permitted for PERSONAL USE ONLY and may not be distributed.		*/
-/* Send all changes/modifications/bugfixes to the above addresses.			*/
-/*											*/
-/* This software is provided AS-IS and has no warranty or guarantee of any type.	*/
-/*											*/
-/*											*/
-/* Id: $Id: sentry_io.c,v 1.2 1998/03/11 04:24:13 crowland Exp crowland $	*/
-/****************************************************************************************/
+/************************************************************************/
+/*                                                                      */
+/* Sentry - Port Scan Detector IO Library                               */
+/*                                                                      */
+/* Author: Craig H. Rowland <crowland@psionic.com>                      */
+/*                          <crowland@bipolar.net>                      */
+/* Created: 10-12-97                                                    */
+/* Modified: 05-25-98                                                   */
+/*                                                                      */
+/* Send all changes/modifications/bugfixes to the above addresses.      */
+/*                                                                      */
+/*                                                                      */
+/* This software is Copyright(c) 1997-98 Craig H. Rowland               */
+/*                                                                      */
+/* Disclaimer:								*/
+/*									*/
+/* All software distributed by Craig H. Rowland ("the author") and	*/
+/* Psionic Systems is distributed AS IS and carries NO WARRANTY or	*/
+/* GUARANTEE OF ANY KIND. End users of the software acknowledge that	*/
+/* they will not hold the author, Psionic Systems, and any employer of	*/
+/* the author liable for failure or non-function of a software		*/
+/* product. YOU ARE USING THIS PRODUCT AT YOUR OWN RISK			*/
+/*									*/
+/* Licensing restrictions apply. See the license that came with this	*/
+/* file for more information or visit http://www.psionic.com for more	*/
+/* information.								*/
+/*                                                                      */
+/* Id: $Id: sentry_io.c,v 1.13 1998/05/28 13:19:15 crowland Exp crowland $*/
+/************************************************************************/
 
 #include "sentry.h"
 #include "sentry_io.h"
@@ -33,7 +43,8 @@ void abacusLog(char *logentry)
 
 void abacusExit(int status)
 {
-   abacusLog("adminalert: abacus is shutting down\n");
+   abacusLog("securityalert: sentry is shutting down\n");
+   abacusLog("adminalert: sentry is shutting down\n");
    exit(status);
 }
 
@@ -61,7 +72,8 @@ int abacusDaemonSeed(void)
    signal(SIGTERM, abacusExit);
    signal(SIGABRT,abacusExit);
    signal(SIGURG, abacusExit);
-
+   signal(SIGKILL, abacusExit);
+   
    if((childpid = fork()) < 0)
       return(ERROR);
    else if(childpid > 0)
@@ -175,7 +187,7 @@ int abacusCheckConfig(void)
 
 /* This writes out blocked hosts to the blocked file. It adds the hostname */
 /* time stamp, and port connection that was acted on */
-int abacusWriteBlocked(char *target, int port)
+int abacusWriteBlocked(char *target, char *resolvedHost, int port)
 {
    FILE *output;
    int blockedStatus=TRUE, historyStatus=TRUE;
@@ -206,9 +218,9 @@ int abacusWriteBlocked(char *target, int port)
       else
       {
 
-	 fprintf(output,"%ld - %d/%d/%d %d:%d:%d Host: %s Port: %d Blocked\n",
+	 fprintf(output,"%ld - %02d/%02d/%02d %02d:%02d:%02d Host: %s/%s Port: %d Blocked\n",
 	    current_time, tmptr->tm_mon+1, tmptr->tm_mday, tmptr->tm_year,
-	    tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, target, port);
+	    tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, resolvedHost, target, port);
 	 fclose(output);
 	 blockedStatus=TRUE;
       }
@@ -237,10 +249,9 @@ int abacusWriteBlocked(char *target, int port)
       }
       else
       {
-
-	 fprintf(output,"%ld - %d/%d/%d %d:%d:%d Host: %s Port: %d\n",
+	 fprintf(output,"%ld - %02d/%02d/%02d %02d:%02d:%02d Host: %s/%s Port: %d Blocked\n",
 	    current_time, tmptr->tm_mon+1, tmptr->tm_mday, tmptr->tm_year,
-	    tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, target, port);
+	    tmptr->tm_hour, tmptr->tm_min, tmptr->tm_sec, resolvedHost, target, port);
 	 fclose(output);
 	 historyStatus=TRUE;
       }
@@ -297,6 +308,13 @@ int abacusConfigTokenRetrieve(char *token, char *configToken)
                   /* search for the token */
 		  if(strstr(buffer, token) != (char)NULL)
 		  { /* cut off the '=' and send it back */
+                     if(strstr(buffer, "\"") == (char)NULL)
+                     {
+                     	snprintf(logbuffer, MAXBUF, "adminalert: Quotes missing from %s token. Option skipped\n", token);
+                	abacusLog(logbuffer);
+                        fclose(config);
+                        return(FALSE);
+                     }
 		     strncpy(buffer ,strstr(buffer,"\"")+1, MAXBUF);
 		     buffer[MAXBUF-1]='\0';
                      /* strip off unprintables/linefeeds (if any) */
@@ -335,10 +353,11 @@ int abacusConfigTokenRetrieve(char *token, char *configToken)
 int abacusBindSocket(int sockfd, struct sockaddr_in client,
    struct sockaddr_in server, int port)
 {
-   char status[MAXBUF];
 #ifdef DEBUG
-   snprintf(status, MAXBUF, "debug: abacusBindSocket: Binding to port: %d\n", port);
-   abacusLog(status);
+   char logbuffer[MAXBUF];
+
+   snprintf(logbuffer, MAXBUF, "debug: abacusBindSocket: Binding to port: %d\n", port);
+   abacusLog(logbuffer);
 #endif
 
    bzero((char *) &server, sizeof(server));
@@ -382,8 +401,6 @@ int abacusOpenTCPSocket(void)
 }
 
 
-
-
 /* Open a UDP Socket */
 int abacusOpenUDPSocket(void)
 {
@@ -399,6 +416,37 @@ int abacusOpenUDPSocket(void)
       return(sockfd);
 }
 
+#ifdef SUPPORT_STEALTH
+/* Open a RAW TCPSocket */
+int abacusOpenRAWTCPSocket(void)
+{
+   int sockfd;
+
+#ifdef DEBUG
+   abacusLog("debug: abacusOpenRAWTCPSocket: opening RAW TCP socket\n");
+#endif
+
+   if((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_TCP)) < 0)
+      return(ERROR);
+   else
+      return(sockfd);
+}
+
+/* Open a RAW UDP Socket */
+int abacusOpenRAWUDPSocket(void)
+{
+   int sockfd;
+
+#ifdef DEBUG
+   abacusLog("debug: abacusOpenRAWUDPSocket: opening RAW UDP socket\n");
+#endif
+
+   if((sockfd = socket(AF_INET, SOCK_RAW, IPPROTO_UDP)) < 0)
+      return(ERROR);
+   else
+      return(sockfd);
+}
+#endif
 
 /* This will use a system() call to change the route of the target host to */
 /* a dead IP address on your LOCAL SUBNET. */
@@ -663,8 +711,6 @@ int abacusIsBlocked(char *target)
       snprintf(logbuffer, MAXBUF, "debug: abacusIsBlocked: Opening block file: %s \n", configToken);
       abacusLog(logbuffer);
 #endif
-
-
       if((input=fopen(configToken, "r")) == NULL)
       {
 	 snprintf(logbuffer, MAXBUF, "adminalert: ERROR: Cannot open blocked file: %s for reading. Will create.\n",	    configToken);
@@ -685,13 +731,13 @@ int abacusIsBlocked(char *target)
 	       return(TRUE);
 	    }
 	 }
+       fclose(input);
       }
 #ifdef DEBUG
       snprintf(logbuffer, MAXBUF, "debug: abacusIsBlocked: Host: %s NOT found in blocked file\n", target);
       abacusLog(logbuffer);
 #endif
    }
-   fclose(input);
    return(FALSE);
 }
 
@@ -739,14 +785,14 @@ int abacusSubstString(char *target, char *victim, char *string)
       }
 
       /* replace the space we nuked */
-      strncat(tempString, " ");
+      strncat(tempString, " ", MAXBUF);
 
       while((tempPtr=(char *)strtok(NULL," ")) != (char)NULL)
       {
 	 if(stringLength > MAXBUF)
 	    break;
 
-	 strncat(tempString," ");
+	 strncat(tempString," ", MAXBUF);
 	 numberOfSubst++;
 
             /* if the token has our victim string in it, then replace, otherwise copy */
@@ -828,4 +874,26 @@ int abacusDoBlockTCP(void)
 }
 
 
+/* snprintf for NEXTSTEP */
+#ifdef HAS_NO_SNPRINTF
+int snprintf(char *str, size_t n, char const *fmt, ...) {
+    va_list ap;
+    FILE f;
+
+    if (n > INT_MAX) {
+       n = INT_MAX;
+    }
+    va_start(ap, fmt);
+    f._file = EOF;
+    f._flag = _IOWRT | _IOSTRG;
+    f._base = f._ptr = str;
+    f._bufsiz = f._cnt = n ? n - 1 : 0;
+    (void)vfprintf(&f, fmt, ap);
+    va_end(ap);
+    if (n) {
+       *f._ptr = '\0';
+    }
+    return (f._ptr - str);
+ }
+#endif
 
